﻿using Npoi.Core.OpenXml4Net.OPC;
using Npoi.Core.OpenXml4Net.Util;
using System;
using System.IO;
using System.Xml;
using System.Xml.Linq;
using System.Xml.Serialization;

namespace Npoi.Core.OpenXmlFormats.Spreadsheet
{
    [Serializable]
    [XmlType(Namespace = "http://schemas.openxmlformats.org/spreadsheetml/2006/main")]
    public class CT_Hyperlink
    {
        private string refField = null;

        private string idField = null; // this and the other ones are optional

        private string locationField = null;

        private string tooltipField = null;

        private string displayField = null;

        [XmlAttribute("ref")]
        public string @ref
        {
            get
            {
                return this.refField;
            }
            set
            {
                this.refField = value;
            }
        }

        [XmlAttribute(Namespace = "http://schemas.openxmlformats.org/officeDocument/2006/relationships")]
        public string id
        {
            get
            {
                return this.idField;
            }
            set
            {
                this.idField = value;
            }
        }

        [XmlAttribute]
        public string location
        {
            get
            {
                return this.locationField;
            }
            set
            {
                this.locationField = value;
            }
        }

        [XmlAttribute]
        public string tooltip
        {
            get
            {
                return this.tooltipField;
            }
            set
            {
                this.tooltipField = value;
            }
        }

        [XmlAttribute]
        public string display
        {
            get
            {
                return this.displayField;
            }
            set
            {
                this.displayField = value;
            }
        }

        public static CT_Hyperlink Parse(XElement node, XmlNamespaceManager namespaceManager)
        {
            if (node == null)
                return null;
            CT_Hyperlink ctObj = new CT_Hyperlink();
            ctObj.@ref = XmlHelper.ReadString(node.Attribute("ref"));
            ctObj.id = XmlHelper.ReadString(node.Attribute((XNamespace)PackageNamespaces.SCHEMA_RELATIONSHIPS + "id"));
            ctObj.location = XmlHelper.ReadString(node.Attribute("location"));
            ctObj.tooltip = XmlHelper.ReadString(node.Attribute("tooltip"));
            ctObj.display = XmlHelper.ReadString(node.Attribute("display"));
            return ctObj;
        }

        internal void Write(StreamWriter sw, string nodeName)
        {
            sw.Write(string.Format("<{0}", nodeName));
            XmlHelper.WriteAttribute(sw, "ref", this.@ref);
            XmlHelper.WriteAttribute(sw, "r:id", this.id);
            XmlHelper.WriteAttribute(sw, "location", this.location);
            XmlHelper.WriteAttribute(sw, "tooltip", this.tooltip);
            XmlHelper.WriteAttribute(sw, "display", this.display);
            sw.Write(">");
            sw.Write(string.Format("</{0}>", nodeName));
        }
    }
}